package org.fax4j.spi.process;

import java.text.MessageFormat;
import org.fax4j.FaxException;
import org.fax4j.FaxJob;
import org.fax4j.FaxJobStatus;
import org.fax4j.common.FaxActionType;
import org.fax4j.spi.AbstractFax4JClientSpi;
import org.fax4j.util.ProcessExecutorHelper;
import org.fax4j.util.ProcessExecutorHelper.ProcessOutput;
import org.fax4j.util.ReflectionHelper;
import org.fax4j.util.SpiUtil;

/**
 * This class implements the fax client service provider interface.<br>
 * This implementation will invoke the requests by executing an external process (executable/script) that enables
 * to invoke the fax action.<br>
 * This class will use request templates to construct the process command.<br> 
 * The template parameters are updated with the values from the fax4j configuration or FaxJob data.<br>
 * All template parameters are defined as ${templatename} variables, for example: myexe.exe -target <b>${target.address}</b> -file <b>${file}</b><br>
 * <br>
 * Below table describes the template parameters.<br>
 * <b>Template Parameters:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Template Parameter</td>
 * 		<td>Description</td>
 * </tr>
 * 	<tr>
 * 		<td>file</td>
 * 		<td>The fax job file path</td>
 * </tr>
 * 	<tr>
 * 		<td>target.address</td>
 * 		<td>The fax job target address</td>
 * </tr>
 * 	<tr>
 * 		<td>target.name</td>
 * 		<td>The fax job target name</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.name</td>
 * 		<td>The fax job sender name</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.fax.number</td>
 * 		<td>The fax job sender fax number</td>
 * </tr>
 * 	<tr>
 * 		<td>sender.email</td>
 * 		<td>The fax job sender email</td>
 * </tr>
 * </table>
 * <br>
 * In case there is no template defined for a certain fax action, this class will throw an UnsupportedOperationException exception.
 * <br>
 * The configuration of the fax4j framework is made up of 3 layers.<br>
 * The configuration is based on simple properties.<br>
 * Each layer overrides the lower layers by adding/changing the property values.<br>
 * The first layer is the internal fax4j.properties file located in the fax4j jar.<br>
 * This layer contains the preconfigured values for the fax4j framework and can be changed
 * by updating these properties in the higher layers.<br>
 * The second layer is the external fax4j.properties file that is located on the classpath.<br>
 * This file is optional and provides the ability to override the internal configuration for the
 * entire fax4j framework.<br>
 * The top most layer is the optional java.util.Properties object provided by the external classes
 * when creating a new fax client.<br>
 * These properties enable to override the configuration of the lower 2 layers.<br>
 * <br>
 * <b>SPI Status (Draft, Beta, Stable): </b>Stable<br>
 * <br>
 * Below table describes the configuration values relevant for this class.<br>
 * <b>Configuration:</b>
 * <table border="1">
 * 	<tr>
 * 		<td>Name</td>
 * 		<td>Description</td>
 * 		<td>Preconfigured Value</td>
 * 		<td>Default Value</td>
 * 		<td>Mandatory</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.submit.template.command</td>
 * 		<td>The submit fax job template command.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.suspend.template.command</td>
 * 		<td>The suspend fax job template command.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.resume.template.command</td>
 * 		<td>The resume fax job template command.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.cancel.template.command</td>
 * 		<td>The cancel fax job template command.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.get.status.template.command</td>
 * 		<td>The get status fax job template command.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.output.validator</td>
 * 		<td>The process output validator class name.</td>
 * 		<td>org.fax4j.spi.process.ExitCodeProcessOutputValidator</td>
 * 		<td>org.fax4j.spi.process.ExitCodeProcessOutputValidator</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.output.handler</td>
 * 		<td>The process output handler class name.</td>
 * 		<td>none</td>
 * 		<td>none</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.use.windows.command.prefix</td>
 * 		<td>True to add a windows command prefix to the command template.</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 		<td>false</td>
 * 	</tr>
 * 	<tr>
 * 		<td>org.fax4j.spi.process.windows.command.prefix</td>
 * 		<td>True to add a windows command prefix to the command template.</td>
 * 		<td>rundll32 SHELL32.DLL,ShellExec_RunDLL</td>
 * 		<td>rundll32 SHELL32.DLL,ShellExec_RunDLL</td>
 * 		<td>false</td>
 * 	</tr>
 * </table>
 * <br>
 * <b>Dependencies:</b><br>
 * <ul>
 * 	<li>External executable/script must be available
 * </ul>
 * <br>
 * 
 * @author 	Sagie Gur-Ari
 * @version 1.05
 * @since	0.21e
 */
public class ProcessFaxClientSpi extends AbstractFax4JClientSpi
{
	/**The process output validator*/
	private ProcessOutputValidator processOutputValidator;
	/**The process output handler*/
	private ProcessOutputHandler processOutputHandler;
	/**The use windows command prefix flag*/
	protected boolean useWindowsCommandPrefix;
	/**The windows command prefix value*/
	private String windowsCommandPrefix;
	/**The property part*/
	private String propertyPart;
	
	/**
	 * This class holds the SPI configuration constants.
	 * 
	 * @author 	Sagie Gur-Ari
	 * @version 1.01
	 * @since	0.21e
	 */
	public static class FaxClientSpiConfigurationConstants
	{
		/**
		 * This is the class constructor.
		 */
		protected FaxClientSpiConfigurationConstants()
		{
			super();
		}

		/**The submit fax job template command property key*/
		public static final String SUBMIT_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.submit.template.command";
		/**The suspend fax job template command property key*/
		public static final String SUSPEND_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.suspend.template.command";
		/**The resume fax job template command property key*/
		public static final String RESUME_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.resume.template.command";
		/**The cancel fax job template command property key*/
		public static final String CANCEL_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.cancel.template.command";
		/**The get status fax job template command property key*/
		public static final String GET_STATUS_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.get.status.template.command";
		/**The process output validator property key*/
		public static final String PROCESS_OUTPUT_VALIDATOR_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.output.validator";
		/**The process output handler property key*/
		public static final String PROCESS_OUTPUT_HANDLER_PRE_FORMAT_PROPERTY_KEY="org.fax4j.spi.{0}.output.handler";
		/**The use windows command prefix property key*/
		public static final String USE_WINDOWS_PREFIX_PROPERTY_KEY="org.fax4j.spi.process.use.windows.command.prefix";
		/**The windows command prefix property key*/
		public static final String WINDOWS_PREFIX_PROPERTY_KEY="org.fax4j.spi.process.windows.command.prefix";
		/**The windows execute prefix*/
		protected static final String WINDOWS_EXECUTE_PREFIX_DEFAULT_VALUE="rundll32 SHELL32.DLL,ShellExec_RunDLL";
		/**The property part default value*/
		protected static final String PROPERTY_PART_DEFAULT_VALUE="process";
	}

	/**
	 * This is the default constructor.
	 */
	public ProcessFaxClientSpi()
	{
		super();
	}

	/**
	 * This function initializes the fax client SPI.
	 */
	@Override
	protected void initializeImpl()
	{
		//get use windows command prefix flag
		String value=this.getConfigurationValue(FaxClientSpiConfigurationConstants.USE_WINDOWS_PREFIX_PROPERTY_KEY);
		this.useWindowsCommandPrefix=Boolean.parseBoolean(value);
		
		//get windows command prefix
		this.windowsCommandPrefix=this.getConfigurationValue(FaxClientSpiConfigurationConstants.WINDOWS_PREFIX_PROPERTY_KEY);
		if(this.windowsCommandPrefix==null)
		{
			this.windowsCommandPrefix=FaxClientSpiConfigurationConstants.WINDOWS_EXECUTE_PREFIX_DEFAULT_VALUE;
		}

		//get property part
		this.propertyPart=this.getPropertyPart();
		if((this.propertyPart==null)||(this.propertyPart.length()==0))
		{
			this.propertyPart=FaxClientSpiConfigurationConstants.PROPERTY_PART_DEFAULT_VALUE;
		}

		//format property key
		String propertyKey=MessageFormat.format(FaxClientSpiConfigurationConstants.PROCESS_OUTPUT_VALIDATOR_PRE_FORMAT_PROPERTY_KEY,new Object[]{this.propertyPart});
		
		//get process output validator
		String className=this.getConfigurationValue(propertyKey);
		if(className==null)
		{
			className=ExitCodeProcessOutputValidator.class.getName();
		}
		try
		{
			//create new instance
			this.processOutputValidator=(ProcessOutputValidator)ReflectionHelper.createInstance(className);
		}
		catch(Exception exception)
		{
			throw new FaxException("Unable to create a new process output validator for class name: "+className,exception);
		}

		//format property key
		propertyKey=MessageFormat.format(FaxClientSpiConfigurationConstants.PROCESS_OUTPUT_HANDLER_PRE_FORMAT_PROPERTY_KEY,new Object[]{this.propertyPart});
		
		//get process output handler
		className=this.getConfigurationValue(propertyKey);
		if(className!=null)
		{
			try
			{
				//create new instance
				this.processOutputHandler=(ProcessOutputHandler)ReflectionHelper.createInstance(className);
			}
			catch(Exception exception)
			{
				throw new FaxException("Unable to create a new process output handler for class name: "+className,exception);
			}
		}
	}
	
	/**
	 * This function returns the property part.
	 * 
	 * @return	The property part
	 */
	protected String getPropertyPart()
	{
		return FaxClientSpiConfigurationConstants.PROPERTY_PART_DEFAULT_VALUE;
	}

	/**
	 * This function will submit a new fax job.<br>
	 * The fax job ID may be populated by this method in the provided
	 * fax job object.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void submitFaxJobImpl(FaxJob faxJob)
	{
		//create command
		String command=this.createSubmitFaxJobProcessCommand(faxJob);
		
		//execute command
		this.executeProcess(faxJob,command,FaxActionType.SUBMIT_FAX_JOB);
	}
	
	/**
	 * This function will suspend an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void suspendFaxJobImpl(FaxJob faxJob)
	{
		//create command
		String command=this.createSuspendFaxJobProcessCommand(faxJob);
		
		//execute command
		this.executeProcess(faxJob,command,FaxActionType.SUSPEND_FAX_JOB);
	}
	
	/**
	 * This function will resume an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void resumeFaxJobImpl(FaxJob faxJob)
	{
		//create command
		String command=this.createResumeFaxJobProcessCommand(faxJob);
		
		//execute command
		this.executeProcess(faxJob,command,FaxActionType.RESUME_FAX_JOB);
	}
	
	/**
	 * This function will cancel an existing fax job.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 */
	@Override
	protected void cancelFaxJobImpl(FaxJob faxJob)
	{
		//create command
		String command=this.createCancelFaxJobProcessCommand(faxJob);
		
		//execute command
		this.executeProcess(faxJob,command,FaxActionType.CANCEL_FAX_JOB);
	}
	
	/**
	 * This function returns the fax job status.<br>
	 * Not all SPIs support extraction of the fax job status.<br>
	 * In case the SPI is unable to extract or does not support extracting
	 * of the fax job status, it will return the UNKNOWN status.
	 * 
	 * @param 	faxJob
	 * 			The fax job object containing the needed information
	 * @return	The fax job status
	 */
	@Override
	protected FaxJobStatus getFaxJobStatusImpl(FaxJob faxJob)
	{
		//create command
		String command=this.createGetStatusFaxJobProcessCommand(faxJob);

		FaxJobStatus faxJobStatus=FaxJobStatus.UNKNOWN;
		if(command!=null)
		{
			//execute command
			ProcessOutput processOutput=this.executeProcess(faxJob,command,FaxActionType.GET_FAX_JOB);
			
			//get fax job status
			faxJobStatus=this.getFaxJobStatus(processOutput);
		}

		return faxJobStatus;
	}
	
	/**
	 * This function extracts the fax job status from the process output.
	 * 
	 * @param	processOutput
	 * 			The process output
	 * @return	The fax job status
	 */
	protected FaxJobStatus getFaxJobStatus(ProcessOutput processOutput)
	{
		FaxJobStatus faxJobStatus=null;
		if(this.processOutputHandler!=null)
		{
			//get fax job status
			faxJobStatus=this.processOutputHandler.getFaxJobStatus(this,processOutput);
		}
		
		//if unable to get the status, set to unknown
		if(faxJobStatus==null)
		{
			faxJobStatus=FaxJobStatus.UNKNOWN;
		}

		return faxJobStatus;
	}
	
	/**
	 * Executes the process and returns the output.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	command
	 * 			The command to execute
	 * @param	faxActionType
	 * 			The fax action type
	 * @return	The process output
	 */
	protected ProcessOutput executeProcess(FaxJob faxJob,String command,FaxActionType faxActionType)
	{
		if(command==null)
		{
			this.throwUnsupportedException();
		}

		//update command
		String updatedCommand=command;
		if(this.useWindowsCommandPrefix)
		{
			//init buffer
			StringBuilder buffer=new StringBuilder(updatedCommand.length()+this.windowsCommandPrefix.length()+1);
			
			//update command
			buffer.append(this.windowsCommandPrefix);
			buffer.append(" ");
			buffer.append(updatedCommand);
			updatedCommand=buffer.toString();
		}

		ProcessOutput processOutput=null;
		try
		{
			//execute process
			processOutput=ProcessExecutorHelper.executeProcess(this,updatedCommand);
		}
		catch(RuntimeException exception)
		{
			throw exception;
		}
		catch(Exception exception)
		{
			throw new FaxException("Error while invoking process.",exception);
		}

		//validate output (if not valid, an exception should be thrown)
		this.validateProcessOutput(processOutput,faxActionType);
		
		//update fax job
		this.updateFaxJob(faxJob,processOutput,faxActionType);
		
		return processOutput;
	}
	
	/**
	 * This function formats the provided template.
	 * 
	 * @param 	template
	 * 			The template
	 * @param 	faxJob
	 * 			The fax job object
	 * @return	The formatted template
	 */
	protected String formatTemplate(String template,FaxJob faxJob)
	{
		return SpiUtil.formatTemplate(template,faxJob,null,false,true);
	}
	
	/**
	 * This function returns the template name.
	 * 
	 * @param	templateName
	 * 			The template name
	 * @return	The updated template name
	 */
	protected String getFormattedTemplateName(String templateName)
	{
		//format template name
		String updatedTemplateName=MessageFormat.format(templateName,new Object[]{this.propertyPart});
		
		return updatedTemplateName;
	}
	
	/**
	 * This function returns the template for the given template name.
	 * 
	 * @param	templateName
	 * 			The template name
	 * @return	The template
	 */
	protected String getTemplate(String templateName)
	{
		//format template name
		String updatedTemplateName=this.getFormattedTemplateName(templateName);

		//get template
		String template=this.getConfigurationValue(updatedTemplateName);

		return template;
	}
	
	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param	templateName
	 * 			The template name
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createProcessCommand(String templateName,FaxJob faxJob)
	{
		//get template
		String template=this.getTemplate(templateName);
		String command=null;
		if(template!=null)
		{
			//format template
			command=this.formatTemplate(template,faxJob);
		}

		return command;
	}
	
	/**
	 * This function validates the process output for errors.<br>
	 * If not valid, an exception should be thrown.
	 * 
	 * @param	processOutput
	 * 			The process output to validate
	 * @param	faxActionType
	 * 			The fax action type
	 */
	protected void validateProcessOutput(ProcessOutput processOutput,FaxActionType faxActionType)
	{
		this.processOutputValidator.validateProcessOutput(this,processOutput,faxActionType);
	}

	/**
	 * Updates the fax job based on the data from the process output.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @param 	processOutput
	 * 			The process output
	 * @param	faxActionType
	 * 			The fax action type
	 */
	protected void updateFaxJob(FaxJob faxJob,ProcessOutput processOutput,FaxActionType faxActionType)
	{
		if(this.processOutputHandler!=null)
		{
			//update fax job
			this.processOutputHandler.updateFaxJob(this,faxJob,processOutput,faxActionType);
		}
	}
	
	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createSubmitFaxJobProcessCommand(FaxJob faxJob)
	{
		return this.createProcessCommand(FaxClientSpiConfigurationConstants.SUBMIT_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY,faxJob);
	}

	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createSuspendFaxJobProcessCommand(FaxJob faxJob)
	{
		return this.createProcessCommand(FaxClientSpiConfigurationConstants.SUSPEND_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY,faxJob);
	}

	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createResumeFaxJobProcessCommand(FaxJob faxJob)
	{
		return this.createProcessCommand(FaxClientSpiConfigurationConstants.RESUME_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY,faxJob);
	}
	
	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createCancelFaxJobProcessCommand(FaxJob faxJob)
	{
		return this.createProcessCommand(FaxClientSpiConfigurationConstants.CANCEL_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY,faxJob);
	}
	
	/**
	 * Creates the process command from the fax job data.
	 *  
	 * @param 	faxJob
	 * 			The fax job object
	 * @return 	The process command to execute
	 */
	protected String createGetStatusFaxJobProcessCommand(FaxJob faxJob)
	{
		return this.createProcessCommand(FaxClientSpiConfigurationConstants.GET_STATUS_FAX_JOB_TEMPLATE_PRE_FORMAT_PROPERTY_KEY,faxJob);
	}
}